﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using Persistence;

public static class Extensions
{
    public static void SelectPage(this GridView grid, string dataKeyValue)
    {
        //select the page and row matching the dataKeyValue
        for (int p = 0; p < grid.PageCount; p++)
        {
            grid.PageIndex = p;
            grid.DataBind();

            for (int i = 0; i < grid.DataKeys.Count; i++)
                if (Convert.ToString(grid.DataKeys[i].Value) == dataKeyValue)
                {
                    grid.SelectedIndex = i;
                    grid.DataBind();
                    return;
                }
        }
        //throw new ApplicationException(String.Format("The ID {0} (DataKey value) was found in {1}.", dataKeyValue, grid.ID));
    }

    public static void SetValueIn(this GridViewRow row, string controlName, string value)
    {
        Control control = row.FindControl(controlName);
        if (control == null)
            throw new ApplicationException(String.Format("Unable to find the {0} control in row {1}.", controlName, row.RowIndex));

        if (control is Label)
            ((Label)control).Text = value;
        else if (control is TextBox)
            ((TextBox)control).Text = value;
        else
            throw new NotSupportedException(String.Format("The type of control {0} is not supported in row {1}.", typeof(Control), row.RowIndex));
    }

    public static string GetValueIn(this GridViewRow row, string controlName)
    {
        Control control = row.FindControl(controlName);
        if (control == null)
            throw new ApplicationException(String.Format("Unable to find the {0} control in row {1}.", controlName, row.RowIndex));

		if (control is DropDownList)
			if (((DropDownList)control).SelectedValue == null)
				return "";
			else
				return (((DropDownList)control).SelectedValue);
		else if (control is HiddenField)
			return ((HiddenField)control).Value;
		else if (control is TextBox)
			return ((TextBox)control).Text;
		else if (control is Label)
			return ((Label)control).Text;
		else if (control is CheckBox)
			return ((CheckBox)control).Checked.ToString();

        throw new NotSupportedException(String.Format("The type of control {0} is not supported in row {1}.", control.GetType(), row.RowIndex));
    }

    public static string GetTextIn(this GridViewRow row, string controlName)
    {
        Control control = row.FindControl(controlName);
        if (control == null)
            throw new ApplicationException(String.Format("Unable to find the {0} control in row {1}.", controlName, row.RowIndex));

        if (control is DropDownList)
            if (((DropDownList)control).SelectedItem == null)
                return "";
            else
                return (((DropDownList)control).SelectedItem.Text);
        else if (control is HiddenField)
            return ((HiddenField)control).Value;
        else if (control is TextBox)
            return ((TextBox)control).Text;
        else if (control is Label)
            return ((Label)control).Text;
		else if (control is CheckBox)
			return ((CheckBox)control).Checked.ToString();

        throw new NotSupportedException(String.Format("The type of control {0} is not supported in row {1}.", control.GetType(), row.RowIndex));
    }

    public static DataControlField GetColumn(this GridView grid, string columnName)
    {
        foreach (DataControlField col in grid.Columns)
            if (col.HeaderText.ToLower() == columnName.ToLower())
                return col;
        throw new ApplicationException(String.Format("The {0} column was not found in {1}.", columnName, grid.ID));
    }

    public static int GetColumnIndex(this GridView grid, string columnName)
    {
        return grid.Columns.IndexOf(grid.GetColumn(columnName));
    }

    public static SortDirection GetSortDirection(this GridView grid)
    {
        //add attribute
        if (grid.Attributes["SortDirection"] == null)
            grid.Attributes.Add("SortDirection", SortDirection.Ascending.ToString());

        //get current direction
        string sortDirection = grid.Attributes["SortDirection"];

        //toggle direction
        if (sortDirection == SortDirection.Ascending.ToString())
            sortDirection = SortDirection.Descending.ToString();
        else
            sortDirection = SortDirection.Ascending.ToString();

        //save the last sort
        grid.Attributes["SortDirection"] = sortDirection;

        return (SortDirection)Enum.Parse(typeof(SortDirection), sortDirection);
    }

	public static void BindList(this GridView grid, object list)
	{
		grid.DataSource = list;
		grid.DataBind();
	}

    public static void BindList<T>(this GridView grid, Persistence.PersistentList<T> list)
    {
        grid.DataSource = list;
        grid.DataBind();
    }

	public static int SelectedPrimaryKey(this GridView grid)
	{
		if (grid.SelectedIndex > -1)
			if (grid.SelectedDataKey == null)
				return 0;
				//throw new ApplicationException(String.Format("Data key was not found on selected index {0} in grid {1}.", grid.SelectedIndex, grid.ID));
			else
				return grid.SelectedDataKey.Value.ToInt32();
		else
			return 0;
	}

    public static void FocusAndSelect(this GridViewRow row, string controlName)
    {
        Control control = row.FindControl(controlName);
        if (control == null)
            throw new ApplicationException(String.Format("The {0} control was not found in row {1}.", controlName, row.RowIndex));

        control.Focus();
        if (control is TextBox)
        {
            ((TextBox)control).Attributes.Add("onfocusin", " select();");
        }
    }

	public static void FocusAndSelect(this TextBox textBox)
	{
		textBox.Focus();
		textBox.Attributes.Add("onfocusin", "select();");
	}

    public static void BindList<T>(this ListControl listControl, Persistence.PersistentList<T> list, string textField, string valueField)
    {
		listControl.DataSource = list;
		listControl.DataTextField = textField;
		listControl.DataValueField = valueField;
		listControl.DataBind();
    }

	public static void SelectItem(this ListControl listControl, string currentText, string currentValue)
    {
        ListItem item = listControl.Items.FindByValue(currentValue);

		if (item != null)
		{
			listControl.SelectedIndex = listControl.Items.IndexOf(item);
			return;
		}

        item = listControl.Items.FindByText(currentText);

        if (item == null)
        {
            listControl.Items.Insert(0, new ListItem(currentText, currentValue));
            listControl.SelectedIndex = 0;
        }
        else if (currentText.IsNotEmpty() && currentValue.IsNotEmpty())
            listControl.SelectedIndex = listControl.Items.IndexOf(item);
    }

	public static GridViewRow ClickedRow(this GridView grid, object sender)
	{
		GridViewRow clickedRow = null;
		if (sender is LinkButton)
			clickedRow = ((LinkButton)sender).NamingContainer as GridViewRow;
		else if (sender is Button)
			clickedRow = ((Button)sender).NamingContainer as GridViewRow;
		else if (sender is CheckBox)
			clickedRow = ((CheckBox)sender).NamingContainer as GridViewRow;
		else if (sender is Control)
			clickedRow = ((Control)sender).NamingContainer as GridViewRow;
		else
			throw new NotSupportedException(String.Format("The sender {0} is not yet supported.", sender.GetType().Name));

		if (clickedRow == null)
			throw new InvalidOperationException("Clicked row is null.");

		return clickedRow;
	}
}
