﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Persistence;

namespace doLogic
{
    public class ManagerDivisions : PersistentList<ManagerDivision>
    {
        public Manager Manager { get; protected set; }

        public static ManagerDivisions For(int managerID)
        {
            if (managerID < 1)
                throw new ArgumentOutOfRangeException("ManagerID is required.");

            using (Connection cn = Database.Open())
            {
                ManagerDivisions list = new ManagerDivisions();
                list.Load("ForManager", new { ManagerID = managerID }, cn);

                list.Manager = Manager.Load(managerID, cn);

                foreach (ManagerDivision ld in list)
                {
                    ld.Manager = list.Manager;
                    ld.LoadDivision(cn);
                }
                return list;
            }
        }
    }

    [Persistable("Manager_Divisions", "ID", false)]
    public class ManagerDivision
    {
        public ManagerDivision() { }

        internal static void Store(int divisionID, int managerID, bool isPrimary)
        {
            ManagerDivision md = new ManagerDivision();
            md.ManagerID = managerID;
            md.DivisionID = divisionID;
            md.IsPrimary = isPrimary;
            Database.Store(md);
        }

        internal static void Delete(int divisionID, int managerID)
        {
            ManagerDivision md = new ManagerDivision();
            md.ManagerID = managerID;
            md.DivisionID = divisionID;
            Database.Delete(md);
        }

        public Manager Manager { get; internal set; }
        public Division Division { get; internal set; }

        internal void LoadManager(Connection cn)
        {
            this.Manager = Manager.Load(this.ManagerID, cn);
        }

        internal void LoadDivision(Connection cn)
        {
            this.Division = Division.Load(this.DivisionID, cn);
        }

        #region Manager fields

        public string FullName { get { return this.Manager.FullName; } }

        public string FirstName
        {
            get { return this.Manager.FirstName; }
            set { this.Manager.FirstName = value; }
        }

        public string LastName
        {
            get { return this.Manager.LastName; }
            set { this.Manager.LastName = value; }
        }

        public string MiddleName
        {
            get { return this.Manager.MiddleName; }
            set { this.Manager.MiddleName = value; }
        }

        public string Title
        {
            get { return this.Manager.Title; }
            set { this.Manager.Title = value; }
        }

        public string Phone
        {
            get { return this.Manager.Phone; }
            set { this.Manager.Phone = value.ToNumericOnly(); }
        }

        public string PhoneFormatted
        {
            get { return this.Manager.PhoneFormatted; }
            set { this.Manager.PhoneFormatted = value; }
        }

        public string Extension
        {
            get { return this.Manager.Extension; }
            set { this.Manager.Extension = value; }
        }

        public string Email
        {
            get { return this.Manager.Email; }
            set { this.Manager.Email = value; }
        }
        #endregion

        #region Division fields

        public string DivisionName
        {
            get { return this.Division.DivName; }
            set { this.Division.DivName = value; }
        }

        #endregion

        #region Manager_Location persistable properties

        public int ID { get; set; }
        public int ManagerID { get; set; }
        public int DivisionID { get; set; }
        public bool IsPrimary { get; set; }

        #endregion

        public string IsPrimaryDisplay
        {
            get { return IsPrimary.ToYesNo(); }
            set { IsPrimary = value.ToBoolean(); }
        }
    }
}
